plugin simpleMeshMod ACloneOnCurve
	name:"ACloneOnCurve"
	classID:#(0x7068D378, 0x798D6CC7)
	category:"MCG Clone Tools"
(
	local pluginDefinitionFile -- the plugin definition file
	local pluginInstance -- the plugin instance created from the plugin definition
	local _meshWrapper -- the dotnet wrapper around the stack mesh pointer
	local argsType = dotNetClass "System.Object[]" -- used when creating args array
	local dotnet_ValueToDotNetObject = dotnet.ValueToDotNetObject
	local tmHandler

	-- function wrappers
	local _meshWrapper_SetValue
	local pluginInstance_UpdateMesh

	-- value wrapper local and function wrapper declarations
	local Pick_CurveWrapper, Pick_CurveWrapper_SetValue
	local meshWrapper, meshWrapper_SetValue
	local modifierTransformWrapper, modifierTransformWrapper_SetValue


	parameters main rollout:params
	(
		count ui:count default:5 type:#integer animatable:true
		Pick_Curve ui:Pick_Curve type:#node useNodeOsValidity:true useNodeTmValidity:true
		start_angle ui:start_angle default:0 type:#float animatable:true
		end_angle ui:end_angle default:0 type:#float animatable:true
		_dummy type:#boolean -- this exists to enable easy invalidation of the object        
		pluginGraph type:#filename assettype:#MaxCreationGraph readOnly:true enumAsAsset:true 
        pluginGraphDependencies type:#filenametab assettype:#MaxCreationGraph readOnly:true enumAsAsset:true tabSize:0 tabSizeVariable:true
    ) 

	rollout params "Parameters"
	(
		spinner count "count" range:[1, 100, 5] type:#integer
		pickbutton Pick_Curve "Pick Curve" autodisplay:true type:#node
		spinner start_angle "start angle" range:[-9999, 9999, 0] type:#float
		spinner end_angle "end angle" range:[-9999, 9999, 0] type:#float
	)


	fn initialize = 
	(
		pluginDefinitionFile = @"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Tools\ACloneOnCurve.maxtool"
		local c = dotNetClass "Viper3dsMaxBridge.ModifierPluginInstance"
		local cid = this.classId
		local refMakerWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.ReferenceMaker"
		refMakerWrapper = refMakerWrapperClass._CreateWrapper this
		pluginInstance = c.CreateInstance cid[1] cid[2] pluginDefinitionFile refMakerWrapper
		pluginInstance_UpdateMesh = pluginInstance.UpdateMesh
		local meshWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.Mesh"
		_meshWrapper = meshWrapperClass._CreateWrapper undefined
		_meshWrapper_SetValue = _meshWrapper._SetValue
		pluginGraph = pluginDefinitionFile
		pluginGraphDependencies = #(@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\TransformsOnCurve.maxcompound",@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\TransformOnCurve.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\LerpTangentOnCurve.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\SampleLerp.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\TransformMesh.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\DeformMeshPoints.maxcompound",@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\NodePositionInLocalSpace.maxcompound",@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\NodeTMInLocalSpace.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\OffsetMesh.maxcompound")

		-- value wrapper local initializations
		local Pick_CurveWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.INode"
		Pick_CurveWrapper = Pick_CurveWrapperClass._CreateWrapper (undefined)
		Pick_CurveWrapper_SetValue = Pick_CurveWrapper._SetValue
		local meshWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Mesh"
		meshWrapper = meshWrapperClass._CreateWrapper (undefined)
		meshWrapper_SetValue = meshWrapper._SetValue
		local modifierTransformWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Matrix3"
		modifierTransformWrapper = modifierTransformWrapperClass._CreateWrapper (matrix3 1)
		modifierTransformWrapper_SetValue = modifierTransformWrapper._SetValue

	)

	fn updateMesh = 
	(
		if (tmHandler == undefined) then ( tmHandler = when transform owningNode changes do _dummy = not _dummy )
		if (pluginInstance == undefined) then ( return undefined )
		local args = dotnet_ValueToDotNetObject #(count, Pick_CurveWrapper, start_angle, end_angle, meshWrapper, modifierTransformWrapper) argsType
		pluginInstance_UpdateMesh (currentTime as integer) _meshWrapper args
	)
	
	on modifyMesh do
	(
		try 
		(
			_meshWrapper_SetValue mesh
			Pick_CurveWrapper_SetValue Pick_Curve
			meshWrapper_SetValue mesh
			modifierTransformWrapper_SetValue owningNode.Transform

			updateMesh()
			_meshWrapper_SetValue undefined

		)
		catch()
		ok
	)

	on create do 
	(
		initialize()
	)

	on clone fromObj do 
	(
		initialize()
	)

	on load do 
	(
		initialize()
	)

	on update do 
	(
		initialize()
	)
)
