plugin simpleMeshMod ARepel
	name:"ARepel"
	classID:#(0x366E7B4E, 0x45D7A856)
	category:"Max Creation Graph"
(
	local pluginDefinitionFile -- the plugin definition file
	local pluginInstance -- the plugin instance created from the plugin definition
	local _meshWrapper -- the dotnet wrapper around the stack mesh pointer
	local argsType = dotNetClass "System.Object[]" -- used when creating args array
	local dotnet_ValueToDotNetObject = dotnet.ValueToDotNetObject
	local tmHandler

	-- function wrappers
	local _meshWrapper_SetValue
	local pluginInstance_UpdateMesh

	-- value wrapper local and function wrapper declarations
	local meshWrapper, meshWrapper_SetValue
	local helperWrapper, helperWrapper_SetValue
	local modifierTransformWrapper, modifierTransformWrapper_SetValue
	local boundingBoxWrapper, boundingBoxWrapper_SetValue


	parameters main rollout:params
	(
		helper ui:helper type:#node useNodeOsValidity:true useNodeTmValidity:true
		_influence ui:_influence default:50 type:#float animatable:true
		_dummy type:#boolean -- this exists to enable easy invalidation of the object        
		pluginGraph type:#filename assettype:#MaxCreationGraph readOnly:true enumAsAsset:true 
        pluginGraphDependencies type:#filenametab assettype:#MaxCreationGraph readOnly:true enumAsAsset:true tabSize:0 tabSizeVariable:true
    ) 

	rollout params "Parameters"
	(
		pickbutton helper "helper" autodisplay:true type:#node
		spinner _influence " influence" range:[0, 10000, 50] type:#float
	)


	fn initialize = 
	(
		pluginDefinitionFile = @"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Tools\ARepel.maxtool"
		local c = dotNetClass "Viper3dsMaxBridge.ModifierPluginInstance"
		local cid = this.classId
		local refMakerWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.ReferenceMaker"
		refMakerWrapper = refMakerWrapperClass._CreateWrapper this
		pluginInstance = c.CreateInstance cid[1] cid[2] pluginDefinitionFile refMakerWrapper
		pluginInstance_UpdateMesh = pluginInstance.UpdateMesh
		local meshWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.Mesh"
		_meshWrapper = meshWrapperClass._CreateWrapper undefined
		_meshWrapper_SetValue = _meshWrapper._SetValue
		pluginGraph = pluginDefinitionFile
		pluginGraphDependencies = #(@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\NodePositionOrBoxCenter.maxcompound",@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\NodePositionInLocalSpace.maxcompound",@"C:\Users\digginc\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\NodeTMInLocalSpace.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\BoundingBoxCenter.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\DeformMeshPoints.maxcompound")

		-- value wrapper local initializations
		local meshWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Mesh"
		meshWrapper = meshWrapperClass._CreateWrapper (undefined)
		meshWrapper_SetValue = meshWrapper._SetValue
		local helperWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.INode"
		helperWrapper = helperWrapperClass._CreateWrapper (undefined)
		helperWrapper_SetValue = helperWrapper._SetValue
		local modifierTransformWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Matrix3"
		modifierTransformWrapper = modifierTransformWrapperClass._CreateWrapper (matrix3 1)
		modifierTransformWrapper_SetValue = modifierTransformWrapper._SetValue
		local boundingBoxWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Box3"
		boundingBoxWrapper = boundingBoxWrapperClass._CreateWrapper (box3())
		boundingBoxWrapper_SetValue = boundingBoxWrapper._SetValue

	)

	fn updateMesh = 
	(
		if (tmHandler == undefined) then ( tmHandler = when transform owningNode changes do _dummy = not _dummy )
		if (pluginInstance == undefined) then ( return undefined )
		local args = dotnet_ValueToDotNetObject #(meshWrapper, helperWrapper, modifierTransformWrapper, boundingBoxWrapper, _influence) argsType
		pluginInstance_UpdateMesh (currentTime as integer) _meshWrapper args
	)
	
	on modifyMesh do
	(
		try 
		(
			_meshWrapper_SetValue mesh
			helperWrapper_SetValue helper
			meshWrapper_SetValue mesh
			modifierTransformWrapper_SetValue owningNode.Transform
			boundingBoxWrapper_SetValue boundingBox

			updateMesh()
			_meshWrapper_SetValue undefined

		)
		catch()
		ok
	)

	on create do 
	(
		initialize()
	)

	on clone fromObj do 
	(
		initialize()
	)

	on load do 
	(
		initialize()
	)

	on update do 
	(
		initialize()
	)
)
